@@ -22,9 +22,13 @@ csymamd_default_knobs = [10 1 0] ;
     if (~isempty (strfind (computer, '64')))
 	d = '-largeArrayDims' ;
     end
-    src = '../Source/ccolamd.c ../Source/ccolamd_global.c' ;
     cmd = sprintf ( ...
         'mex -DDLONG -O %s -I../../SuiteSparse_config -I../Include ', d) ;
+    src = '../Source/ccolamd.c ../../SuiteSparse_config/SuiteSparse_config.c' ;
+    if (~(ispc || ismac))
+        % for POSIX timing routine
+        src = [src ' -lrt'] ;
+    end
     eval ([cmd 'ccolamdtestmex.c ' src]) ;
     eval ([cmd 'csymamdtestmex.c ' src]) ;
     fprintf ('Done compiling.\n') ; 
