@@ -3,13 +3,13 @@ typedef struct problem_struct
 {
     cs_dl *A ;
     cs_dl *C ;
-    UF_long sym ;
+    cs_long_t sym ;
     double *x ;
     double *b ;
     double *resid ;
 } problem ;
 
 problem *get_problem (FILE *f, double tol) ;
-UF_long demo2 (problem *Prob) ;
-UF_long demo3 (problem *Prob) ;
+cs_long_t demo2 (problem *Prob) ;
+cs_long_t demo3 (problem *Prob) ;
 problem *free_problem (problem *Prob) ;
