@@ -12,25 +12,19 @@
 
 // Define function pointers and other parameters for a mexFunction
 
-int spqr_mx_config (Int spumoni, cholmod_common *cc)
+int spqr_mx_config (Long spumoni, cholmod_common *cc)
 {
     if (cc == NULL) return (FALSE) ;
 
     // cholmod_l_solve must return a real or zomplex X for MATLAB
     cc->prefer_zomplex = TRUE ;
 
-    // use mxMalloc and related memory management routines
-    cc->malloc_memory  = mxMalloc ;
-    cc->free_memory    = mxFree ;
-    cc->realloc_memory = mxRealloc ;
-    cc->calloc_memory  = mxCalloc ;
-
     // printing and error handling
     if (spumoni == 0)
     {
 	// do not print anything from within CHOLMOD
 	cc->print = -1 ;
-	cc->print_function = NULL ;
+	SuiteSparse_config.printf_func = NULL ;
     }
     else
     {
@@ -38,23 +32,18 @@ int spqr_mx_config (Int spumoni, cholmod_common *cc)
 	//	routines will print a one-line summary of each object printed.
 	// spumoni = 2: also print a short summary of each object.
 	cc->print = spumoni + 2 ;
-	cc->print_function = mexPrintf ;
     }
 
     // error handler
     cc->error_handler = spqr_mx_error ;
     spqr_spumoni = spumoni ;
 
-    // complex arithmetic
-    cc->complex_divide = cholmod_l_divcomplex ;
-    cc->hypotenuse     = cholmod_l_hypot ;
-
 #ifndef NPARTITION
 #if defined(METIS_VERSION)
 #if (METIS_VERSION >= METIS_VER(4,0,2))
     // METIS 4.0.2 uses function pointers for malloc and free
-    METIS_malloc = cc->malloc_memory ;
-    METIS_free   = cc->free_memory ;
+    METIS_malloc = SuiteSparse_config.malloc_func ;
+    METIS_free   = SuiteSparse_config.free_func ;
 #endif
 #endif
 #endif
@@ -94,7 +83,7 @@ void spqr_mx_spumoni
     mexPrintf (
     "\nSuiteSparseQR, version %d.%d.%d (%s), a multithreaded\n"
     "multifrontal sparse QR factorization method, (c) Timothy A. Davis,\n"
-    "University of Florida\n\n",
+    "http://www.suitesparse.com\n\n",
         SPQR_MAIN_VERSION, SPQR_SUB_VERSION, SPQR_SUBSUB_VERSION, SPQR_DATE) ;
 
     // -------------------------------------------------------------------------
@@ -200,9 +189,9 @@ void spqr_mx_spumoni
     "        a coalescing of the column elimination tree with one node per\n"
     "        column in the matrix.  The goal of the merging heuristic is to\n"
     "        create a task graph whose leaf nodes have flop counts >=\n"
-    "        max ((total flops) / opts.task, opts.small).  If opts.task <= 1,\n"
+    "        max ((total flops)/opts.grain, opts.small).  If opts.grain <= 1,\n"
     "        then no TBB parallelism is exploited.  The current default is\n"
-    "        opts.task=1 because in the current version of TBB and OpenMP,\n"
+    "        opts.grain=1 because in the current version of TBB and OpenMP,\n"
     "        TBB parallelism conflicts with BLAS OpenMP-based parallelism.\n"
     "        This will be resolved in a future version.\n"
     "        opts.nthreads is the number of threads that TBB should use.\n"
@@ -223,9 +212,9 @@ void spqr_mx_spumoni
     mexPrintf ("    rank(A) estimate: %ld\n",             cc->SPQR_istat [4]) ;
     mexPrintf ("    # of column singletons: %ld\n",       cc->SPQR_istat [5]) ;
     mexPrintf ("    # of singleton rows: %ld\n",          cc->SPQR_istat [6]) ;
-    mexPrintf ("    upper bound on flop count: %g\n",     cc->SPQR_xstat [0]
-        * (is_complex ? 4 : 1)) ;
-    mexPrintf ("    column 2-norm tolerance used: %g\n",  cc->SPQR_xstat [1]) ;
+    mexPrintf ("    upper bound on flop count: %g\n",
+        cc->SPQR_flopcount_bound * (is_complex ? 4 : 1)) ;
+    mexPrintf ("    column 2-norm tolerance used: %g\n",  cc->SPQR_tol_used) ;
     mexPrintf ("    actual memory usage: %g (bytes)\n",
         ((double) cc->memory_usage)) ;
 
@@ -247,7 +236,7 @@ static int get_option
 
     // outputs:
     double *x,                  // double value of the field, if present
-    Int *x_present,             // true if x is present
+    Long *x_present,            // true if x is present
     char **s,                   // char value of the field, if present;
                                 // must be mxFree'd by caller when done
 
@@ -255,7 +244,7 @@ static int get_option
     cholmod_common *cc
 )
 {
-    Int f ;
+    Long f ;
     mxArray *p ;
 
     if (cc == NULL) return (FALSE) ;
@@ -341,14 +330,14 @@ int spqr_mx_get_options
 (
     const mxArray *mxopts,
     spqr_mx_options *opts,
-    Int m,
+    Long m,
     int nargout,
     cholmod_common *cc
 )
 {
     double x ;
     char *s ;
-    Int x_present ;
+    Long x_present ;
 
     if (cc == NULL) return (FALSE) ;
 
@@ -357,7 +346,7 @@ int spqr_mx_get_options
     // -------------------------------------------------------------------------
 
     get_option (mxopts, "spumoni", &x, &x_present, NULL, cc) ;
-    opts->spumoni = x_present ? ((Int) x) : SPUMONI ;
+    opts->spumoni = x_present ? ((Long) x) : SPUMONI ;
     spqr_spumoni = opts->spumoni ;
 
     // -------------------------------------------------------------------------
@@ -365,7 +354,7 @@ int spqr_mx_get_options
     // -------------------------------------------------------------------------
 
     get_option (mxopts, "econ", &x, &x_present, NULL, cc) ;
-    opts->econ = x_present ? ((Int) x) : m ;
+    opts->econ = x_present ? ((Long) x) : m ;
 
     // -------------------------------------------------------------------------
     // tol: a double with default computed later
@@ -562,20 +551,20 @@ int spqr_mx_get_options
 
 static int put_values
 (
-    Int nz,
+    Long nz,
     mxArray *A,
     double *Ax,         // complex case: size 2*nz and freed on return,
                         // real case: size nz, not freed on return.
-    Int is_complex,
+    Long is_complex,
     cholmod_common *cc
 )
 {
-    Int imag_all_zero = TRUE ;
+    Long imag_all_zero = TRUE ;
 
     if (is_complex)
     {
         // A is complex, stored in interleaved form; split it for MATLAB
-        Int k ;
+        Long k ;
         double z, *Ax2, *Az2 ;
         mxFree (mxGetPi (A)) ;
         Ax2 = (double *) cholmod_l_malloc (nz, sizeof (double), cc) ;
@@ -625,7 +614,7 @@ mxArray *spqr_mx_put_sparse
 {
     mxArray *Amatlab ;
     cholmod_sparse *A ;
-    Int nz, is_complex ;
+    Long nz, is_complex ;
 
     A = *Ahandle ;
     is_complex = (A->xtype != CHOLMOD_REAL) ;
@@ -659,8 +648,8 @@ mxArray *spqr_mx_put_sparse
 
 mxArray *spqr_mx_put_dense2
 (
-    Int m,
-    Int n,
+    Long m,
+    Long n,
     double *Ax,         // size nz if real; size 2*nz if complex
     int is_complex,
     cholmod_common *cc
@@ -722,8 +711,8 @@ mxArray *spqr_mx_put_dense
 
 mxArray *spqr_mx_put_permutation
 (
-    Int *P,         // size n permutation vector
-    Int n, 
+    Long *P,        // size n permutation vector
+    Long n, 
     int vector,     // if TRUE, return a vector; otherwise return a matrix
 
     // workspace and parameters
@@ -732,7 +721,7 @@ mxArray *spqr_mx_put_permutation
 {
     mxArray *Pmatlab ;
     double *Ex ;
-    Int *Ep, *Ei, j, k ;
+    Long *Ep, *Ei, j, k ;
 
     if (cc == NULL) return (NULL) ;
 
@@ -750,8 +739,8 @@ mxArray *spqr_mx_put_permutation
     {
         // return E as a permutation matrix
         Pmatlab = mxCreateSparse (n, n, MAX (n,1), mxREAL) ;
-        Ep = (Int *) mxGetJc (Pmatlab) ;
-        Ei = (Int *) mxGetIr (Pmatlab) ;
+        Ep = (Long *) mxGetJc (Pmatlab) ;
+        Ei = (Long *) mxGetIr (Pmatlab) ;
         Ex = mxGetPr (Pmatlab) ;
         for (k = 0 ; k < n ; k++)
         {
@@ -782,13 +771,13 @@ double *spqr_mx_merge_if_complex
     const mxArray *A,
     int make_complex,       // if TRUE, return value is Complex
     // output
-    Int *p_nz,              // number of entries in A
+    Long *p_nz,             // number of entries in A
 
     // workspace and parameters
     cholmod_common *cc
 )
 {
-    Int nz, m, n ;
+    Long nz, m, n ;
     double *X, *Xx, *Xz ;
 
     if (cc == NULL) return (NULL) ;
@@ -800,7 +789,7 @@ double *spqr_mx_merge_if_complex
 
     if (mxIsSparse (A))
     {
-        Int *Ap = (Int *) mxGetJc (A) ;
+        Long *Ap = (Long *) mxGetJc (A) ;
         nz = Ap [n] ;
     }
     else
@@ -811,7 +800,7 @@ double *spqr_mx_merge_if_complex
     {
         // Note the typecast and sizeof (...) intentionally do not match
         X = (double *) cholmod_l_malloc (nz, sizeof (Complex), cc) ;
-        for (Int k = 0 ; k < nz ; k++)
+        for (Long k = 0 ; k < nz ; k++)
         {
             X [2*k  ] = Xx [k] ;
             X [2*k+1] = Xz ? (Xz [k]) : 0 ;
@@ -840,12 +829,12 @@ cholmod_sparse *spqr_mx_get_sparse
     double *dummy 	    // a pointer to a valid scalar double
 )
 {
-    Int *Ap ;
+    Long *Ap ;
     A->nrow = mxGetM (Amatlab) ;
     A->ncol = mxGetN (Amatlab) ;
-    A->p = (Int *) mxGetJc (Amatlab) ;
-    A->i = (Int *) mxGetIr (Amatlab) ;
-    Ap = (Int *) A->p ;
+    A->p = (Long *) mxGetJc (Amatlab) ;
+    A->i = (Long *) mxGetIr (Amatlab) ;
+    Ap = (Long *) A->p ;
     A->nzmax = Ap [A->ncol] ;
     A->packed = TRUE ;
     A->sorted = TRUE ;
@@ -929,26 +918,26 @@ void spqr_mx_get_usage
 (
     mxArray *A,         // mxArray to check
     int tight,          // if true, then nnz(A) must equal nzmax(A)
-    Int *p_usage,       // bytes used
-    Int *p_count,       // # of malloc'd blocks
+    Long *p_usage,      // bytes used
+    Long *p_count,      // # of malloc'd blocks
     cholmod_common *cc
 )
 {
-    Int nz, m, n, nzmax, is_complex, usage, count, *Ap ;
+    Long nz, m, n, nzmax, is_complex, usage, count, *Ap ;
     m = mxGetM (A) ;
     n = mxGetN (A) ;
     is_complex = mxIsComplex (A) ;
     if (mxIsSparse (A))
     {
         nzmax = mxGetNzmax (A) ;
-        Ap = (Int *) mxGetJc (A) ;
+        Ap = (Long *) mxGetJc (A) ;
         nz = MAX (Ap [n], 1) ;
         if (tight && nz != nzmax)
         {
             // This should never occur.
             mexErrMsgIdAndTxt ("QR:internalError", "nnz (A) < nzmax (A)!") ;
         }
-        usage = sizeof (Int) * (n+1 + nz) ;
+        usage = sizeof (Long) * (n+1 + nz) ;
         count = 2 ;
     }
     else
@@ -984,3 +973,131 @@ char *spqr_mx_id (int line)
     sprintf (spqr_mx_debug_string, "QR:Line_%d", line) ;
     return (spqr_mx_debug_string) ;
 }
+
+
+// =============================================================================
+// === spqr_mx_info ============================================================
+// =============================================================================
+
+mxArray *spqr_mx_info       // return a struct with info statistics
+(
+    cholmod_common *cc,
+    double t,               // total time, < 0 if not computed
+    double flops            // flop count, < 0 if not computed
+)
+{
+    Long ninfo ;
+    mxArray *s ;
+
+    const char *info_struct [ ] =
+    {
+        "nnzR_upper_bound",             // 0: nnz(R) bound
+        "nnzH_upper_bound",             // 1: nnz(H) bound
+        "number_of_frontal_matrices",   // 2: nf
+        "number_of_TBB_tasks",          // 3: ntasks
+        "rank_A_estimate",              // 4: rank
+        "number_of_column_singletons",  // 5: n1cols
+        "number_of_singleton_rows",     // 6: n1rows
+        "ordering",                     // 7: ordering used
+        "memory_usage_in_bytes",        // 8: memory usage
+        "flops_upper_bound",            // 9: upper bound on flop count
+                                        //    (excluding backsolve)
+        "tol",                          // 10: column norm tolerance used
+        "number_of_TBB_threads",        // 11: # threads used
+        "norm_E_fro",                   // 12: norm of dropped diag of R
+
+        // compilation options
+        "spqr_compiled_with_TBB",       // 13: compiled with TBB or not
+        "spqr_compiled_with_METIS",     // 14: compiled with METIS or not
+
+        // only if flops >= 0:
+        "analyze_time",                 // 15: analyze time
+        "factorize_time",               // 16: factorize time (and apply Q')
+        "solve_time",                   // 17: R\C backsolve only
+        "total_time",                   // 18: total x=A\b in seconds
+        "flops"                         // 19: actual flops (incl backsolve)
+    } ;
+
+    ninfo = (flops < 0) ? 15 : 20 ;
+
+    s = mxCreateStructMatrix (1, 1, ninfo, info_struct) ;
+
+    for (Long k = 0 ; k <= 6 ; k++)
+    {
+        mxSetFieldByNumber (s, 0, k,
+            mxCreateDoubleScalar ((double) cc->SPQR_istat [k])) ;
+    }
+
+    // get the ordering used.  Note that "default", "best", and "cholmod"
+    // are not among the possible results, since they are meta-orderings
+    // that select among AMD, COLAMD, and/or METIS.
+    mxArray *ord ;
+    switch (cc->SPQR_istat [7])
+    {
+        case SPQR_ORDERING_FIXED:
+        case SPQR_ORDERING_GIVEN: 
+            ord = mxCreateString ("fixed") ;
+            break ;
+        case SPQR_ORDERING_NATURAL:
+            ord = mxCreateString ("natural") ;
+            break ;
+        case SPQR_ORDERING_COLAMD:
+            ord = mxCreateString ("colamd") ;
+            break ;
+        case SPQR_ORDERING_AMD:
+            ord = mxCreateString ("amd") ;
+            break ;
+#ifndef NPARTITION
+        case SPQR_ORDERING_METIS:
+            ord = mxCreateString ("metis") ;
+            break ;
+#endif
+        default:
+            ord = mxCreateString ("unknown") ;
+            break ;
+    }
+    mxSetFieldByNumber (s, 0, 7, ord) ;
+
+    mxSetFieldByNumber (s, 0, 8,
+        mxCreateDoubleScalar ((double) cc->memory_usage)) ;
+    mxSetFieldByNumber (s, 0, 9,
+        mxCreateDoubleScalar (cc->SPQR_flopcount_bound)) ;
+    mxSetFieldByNumber (s, 0, 10, mxCreateDoubleScalar (cc->SPQR_tol_used)) ;
+
+    int nthreads = cc->SPQR_nthreads ;
+    if (nthreads <= 0)
+    {
+        mxSetFieldByNumber (s, 0, 11, mxCreateString ("default"));
+    }
+    else
+    {
+        mxSetFieldByNumber (s, 0, 11, mxCreateDoubleScalar ((double) nthreads));
+    }
+
+    mxSetFieldByNumber (s, 0, 12, mxCreateDoubleScalar (cc->SPQR_norm_E_fro)) ;
+
+#ifdef HAVE_TBB
+    mxSetFieldByNumber (s, 0, 13, mxCreateString ("yes")) ;
+#else
+    mxSetFieldByNumber (s, 0, 13, mxCreateString ("no")) ;
+#endif
+
+#ifndef NPARTITION
+    mxSetFieldByNumber (s, 0, 14, mxCreateString ("yes")) ;
+#else
+    mxSetFieldByNumber (s, 0, 14, mxCreateString ("no")) ;
+#endif
+
+    if (flops >= 0)
+    {
+        mxSetFieldByNumber (s, 0, 15,
+            mxCreateDoubleScalar (cc->SPQR_analyze_time)) ;
+        mxSetFieldByNumber (s, 0, 16,
+            mxCreateDoubleScalar (cc->SPQR_factorize_time)) ;
+        mxSetFieldByNumber (s, 0, 17,
+            mxCreateDoubleScalar (cc->SPQR_solve_time)) ;
+        mxSetFieldByNumber (s, 0, 18, mxCreateDoubleScalar (t)) ;
+        mxSetFieldByNumber (s, 0, 19, mxCreateDoubleScalar (flops)) ;
+    }
+    return (s) ;
+}
